/* eslint-disable no-magic-numbers */
import { apiPath } from '@/redux/apiPath';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import { unwrapResult } from '@reduxjs/toolkit';
import { LAYER_STATE_DEFAULT_DATA, LAYERS_STATE_INITIAL_MOCK } from '@/redux/layers/layers.mock';
import { getLayersForModel } from '@/redux/layers/layers.thunks';
import { layersFixture } from '@/models/fixtures/layersFixture';
import { layerTextsFixture } from '@/models/fixtures/layerTextsFixture';
import { layerRectsFixture } from '@/models/fixtures/layerRectsFixture';
import { layerOvalsFixture } from '@/models/fixtures/layerOvalsFixture';
import { layerLinesFixture } from '@/models/fixtures/layerLinesFixture';
import { LayersState } from './layers.types';
import layersReducer from './layers.slice';

const mockedAxiosClient = mockNetworkNewAPIResponse();

const INITIAL_STATE: LayersState = LAYERS_STATE_INITIAL_MOCK;

describe('layers reducer', () => {
  let store = {} as ToolkitStoreWithSingleSlice<LayersState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('layers', layersReducer);
  });

  it('should match initial state', () => {
    const action = { type: 'unknown' };

    expect(layersReducer(undefined, action)).toEqual(INITIAL_STATE);
  });

  it('should update store after successful getLayers query', async () => {
    mockedAxiosClient.onGet(apiPath.getLayers(1)).reply(HttpStatusCode.Ok, layersFixture);
    mockedAxiosClient
      .onGet(apiPath.getLayerTexts(1, layersFixture.content[0].id))
      .reply(HttpStatusCode.Ok, layerTextsFixture);
    mockedAxiosClient
      .onGet(apiPath.getLayerRects(1, layersFixture.content[0].id))
      .reply(HttpStatusCode.Ok, layerRectsFixture);
    mockedAxiosClient
      .onGet(apiPath.getLayerOvals(1, layersFixture.content[0].id))
      .reply(HttpStatusCode.Ok, layerOvalsFixture);
    mockedAxiosClient
      .onGet(apiPath.getLayerLines(1, layersFixture.content[0].id))
      .reply(HttpStatusCode.Ok, layerLinesFixture);

    const { type } = await store.dispatch(getLayersForModel(1));
    const { data, loading, error } = store.getState().layers[1];
    expect(type).toBe('vectorMap/getLayers/fulfilled');
    expect(loading).toEqual('succeeded');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual({
      layers: [
        {
          details: layersFixture.content[0],
          texts: layerTextsFixture.content,
          rects: layerRectsFixture.content,
          ovals: layerOvalsFixture.content,
          lines: layerLinesFixture.content,
        },
      ],
      layersVisibility: {
        [layersFixture.content[0].layerId]: layersFixture.content[0].visible,
      },
    });
  });

  it('should update store after failed getLayers query', async () => {
    mockedAxiosClient.onGet(apiPath.getLayers(1)).reply(HttpStatusCode.NotFound, []);

    const action = await store.dispatch(getLayersForModel(1));
    const { data, loading, error } = store.getState().layers[1];

    expect(action.type).toBe('vectorMap/getLayers/rejected');
    expect(() => unwrapResult(action)).toThrow(
      "Failed to fetch layers: The page you're looking for doesn't exist. Please verify the URL and try again.",
    );
    expect(loading).toEqual('failed');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual({ layers: [], layersVisibility: {} });
  });

  it('should update store on loading getLayers query', async () => {
    mockedAxiosClient.onGet(apiPath.getLayers(1)).reply(HttpStatusCode.Ok, layersFixture);
    mockedAxiosClient
      .onGet(apiPath.getLayerTexts(1, layersFixture.content[0].id))
      .reply(HttpStatusCode.Ok, layerTextsFixture);
    mockedAxiosClient
      .onGet(apiPath.getLayerRects(1, layersFixture.content[0].id))
      .reply(HttpStatusCode.Ok, layerRectsFixture);
    mockedAxiosClient
      .onGet(apiPath.getLayerOvals(1, layersFixture.content[0].id))
      .reply(HttpStatusCode.Ok, layerOvalsFixture);
    mockedAxiosClient
      .onGet(apiPath.getLayerLines(1, layersFixture.content[0].id))
      .reply(HttpStatusCode.Ok, layerLinesFixture);

    const layersPromise = store.dispatch(getLayersForModel(1));

    const { data, loading } = store.getState().layers[1];
    expect(data).toEqual(LAYER_STATE_DEFAULT_DATA);
    expect(loading).toEqual('pending');

    layersPromise.then(() => {
      const { data: dataPromiseFulfilled, loading: promiseFulfilled } = store.getState().layers[1];

      expect(dataPromiseFulfilled).toEqual({
        layers: [
          {
            details: layersFixture.content[0],
            texts: layerTextsFixture.content,
            rects: layerRectsFixture.content,
            ovals: layerOvalsFixture.content,
            lines: layerLinesFixture.content,
          },
        ],
        layersVisibility: {
          [layersFixture.content[0].layerId]: layersFixture.content[0].visible,
        },
      });
      expect(promiseFulfilled).toEqual('succeeded');
    });
  });
});
